@@ -1,10 +1,9 @@
+use crate::utils::{deserialize_u256, deserialize_u64};
 use ethers_core::{
     types::{transaction::response::Transaction, Address, Bytes, H256, U256, U64},
     utils::keccak256,
 };
-use serde::{de, Deserialize, Serialize, Serializer};
-use serde_json::Value;
-use std::str::FromStr;
+use serde::{Deserialize, Serialize, Serializer};
 
 /// A bundle hash.
 pub type BundleHash = H256;
@@ -228,6 +227,8 @@ pub struct SimulatedTransaction {
     /// The value sent in this transaction.
     #[serde(deserialize_with = "deserialize_u256")]
     pub value: U256,
+    /// The reason this transaction reverted (if it did).
+    pub error: Option<String>,
 }
 
 /// Details of a simulated bundle.
@@ -269,40 +270,6 @@ pub struct SimulatedBundle {
     pub transactions: Vec<SimulatedTransaction>,
 }
 
-fn deserialize_u64<'de, D>(deserializer: D) -> Result<U64, D::Error>
-where
-    D: de::Deserializer<'de>,
-{
-    Ok(match Value::deserialize(deserializer)? {
-        Value::String(s) => {
-            if s.as_str() == "0x" {
-                return Ok(U64::zero());
-            }
-
-            U64::from(u64::from_str(s.as_str()).map_err(de::Error::custom)?)
-        }
-        Value::Number(num) => U64::from(num.as_u64().ok_or(de::Error::custom("Invalid number"))?),
-        _ => return Err(de::Error::custom("wrong type")),
-    })
-}
-
-fn deserialize_u256<'de, D>(deserializer: D) -> Result<U256, D::Error>
-where
-    D: de::Deserializer<'de>,
-{
-    Ok(match Value::deserialize(deserializer)? {
-        Value::String(s) => {
-            if s.as_str() == "0x" {
-                return Ok(U256::zero());
-            }
-
-            U256::from(u64::from_str(s.as_str()).map_err(de::Error::custom)?)
-        }
-        Value::Number(num) => U256::from(num.as_u64().ok_or(de::Error::custom("Invalid number"))?),
-        _ => return Err(de::Error::custom("wrong type")),
-    })
-}
-
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -344,7 +311,8 @@ mod tests {
         "gasUsed": 21000,
         "toAddress": "0x73625f59CAdc5009Cb458B751b3E7b6b48C06f2C",
         "txHash": "0x669b4704a7d993a946cdd6e2f95233f308ce0c4649d2e04944e8299efcaa098a",
-        "value": "0x"
+        "value": "0x",
+        "error": "execution reverted"
       },
       {
         "coinbaseDiff": "10000000000063000",
@@ -382,5 +350,10 @@ mod tests {
         assert_eq!(simulated_bundle.gas_fees, U256::from(126000));
         assert_eq!(simulated_bundle.simulation_block, U64::from(5221585));
         assert_eq!(simulated_bundle.transactions.len(), 2);
+        assert_eq!(
+            simulated_bundle.transactions[0].error,
+            Some("execution reverted".into())
+        );
+        assert_eq!(simulated_bundle.transactions[1].error, None);
     }
 }
